/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __FS_DOT_H__
#define __FS_DOT_H__

#define GFS_ASSERT_TYPE_NONE      (18)
#define GFS_ASSERT_TYPE_SBD       (19)
#define GFS_ASSERT_TYPE_GLOCK     (20)
#define GFS_ASSERT_TYPE_INODE     (21)
#define GFS_ASSERT_TYPE_RGRPD     (22)

#define GFS_ASSERT(x, todo) \
do \
{ \
  if (!(x)) \
  { \
    {todo} \
    gfs_asserti(#x, __FILE__, __LINE__, GFS_ASSERT_TYPE_NONE, NULL); \
 } \
} \
while (0)

#define GFS_ASSERT_SBD(x, sdp, todo) \
do \
{ \
  if (!(x)) \
  { \
    struct gfs_sbd *gfs_assert_sbd = (sdp); \
    {todo} \
    gfs_asserti(#x, __FILE__, __LINE__, GFS_ASSERT_TYPE_SBD, gfs_assert_sbd); \
  } \
} \
while (0)

#define GFS_ASSERT_GLOCK(x, gl, todo) \
do \
{ \
  if (!(x)) \
  { \
    struct gfs_glock *gfs_assert_glock = (gl); \
    {todo} \
    gfs_asserti(#x, __FILE__, __LINE__, GFS_ASSERT_TYPE_GLOCK, gfs_assert_glock); \
  } \
} \
while (0)

#define GFS_ASSERT_INODE(x, ip, todo) \
do \
{ \
  if (!(x)) \
  { \
    struct gfs_inode *gfs_assert_inode = (ip); \
    {todo} \
    gfs_asserti(#x, __FILE__, __LINE__, GFS_ASSERT_TYPE_INODE, gfs_assert_inode); \
  } \
} \
while (0)

#define GFS_ASSERT_RGRPD(x, rgd, todo) \
do \
{ \
  if (!(x)) \
  { \
    struct gfs_rgrpd *gfs_assert_rgrpd = (rgd); \
    {todo} \
    gfs_asserti(#x, __FILE__, __LINE__, GFS_ASSERT_TYPE_RGRPD, gfs_assert_rgrpd); \
  } \
} \
while (0)

extern uint32_t gfs_random_number;
uint32_t gfs_random(void);

uint32_t gfs_hash(const void *data, unsigned int len);
uint32_t gfs_hash_more(const void *data, unsigned int len, uint32_t hash);

void gfs_sort(void *array, unsigned int num, unsigned int size,
	      int (*compare) (void *, void *));

extern volatile int gfs_in_panic;
void gfs_asserti(char *assertion, char *file, unsigned int line,
		 unsigned int type, void *ptr) __attribute__ ((noreturn));

void bitch_about(struct gfs_sbd *sdp, unsigned long *last, char *about);

void *gmalloc(unsigned int size);

struct gfs_glock *gmalloc_glock(struct gfs_sbd *sdp,
				struct gfs_glock_operations *glops);
void gfree_glock(struct gfs_glock *gl);

struct gfs_inode *gmalloc_inode(void);
void gfree_inode(struct gfs_inode *p);

struct gfs_bufdata *gmalloc_bufdata(void);
void gfree_bufdata(struct gfs_bufdata *p);

struct gfs_mh_cache *gmalloc_mh_cache(void);
void gfree_mh_cache(struct gfs_mh_cache *p);

#endif				/*  __FS_DOT_H__  */
