/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __GFS_DOT_H__
#define __GFS_DOT_H__

#define GFS_RELEASE_NAME "v6.0.0"

/*  Includes  */

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <linux/spinlock.h>
#include <asm/atomic.h>
#include <linux/config.h>
#include <linux/locks.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/ctype.h>
#include <linux/string.h>
#include <linux/types.h>

#include "fixed_div64.h"

#include <linux/lm_interface.h>
#include <linux/gfs_ondisk.h>
#include <linux/gfs_ioctl.h>

#include "lvb.h"
#include "incore.h"
#include "fs.h"

#define NO_ST_CREATE     (0)
#define ST_CREATE        (1)

/*  Divide x by y.  Round up if there is a remainder.  */
#define DIV_RU(x, y) (((x) + (y) - 1) / (y))

#define SM2S(var, type, mem) ((type *)((unsigned long)(var) - (unsigned long)(&((type *)NULL)->mem)))

#define GFS_FAST_NAME_SIZE (32)

#define RETRY_MALLOC(do_this, until_this) \
for (;;) \
{ \
  do { do_this; } while (0); \
  if (until_this) \
    break; \
  printk("GFS: out of memory: %s, %u\n", __FILE__, __LINE__); \
  yield();\
}

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#if (BITS_PER_LONG == 64)
#define PRIu64 "lu"
#define PRId64 "ld"
#define PRIo64 "lo"
#define PRIx64 "lx"
#define PRIX64 "lX"
#define SCNu64 "lu"
#define SCNd64 "ld"
#define SCNo64 "lo"
#define SCNx64 "lx"
#define SCNX64 "lX"
#else
#define PRIu64 "Lu"
#define PRId64 "Ld"
#define PRIo64 "Lo"
#define PRIx64 "Lx"
#define PRIX64 "LX"
#define SCNu64 "Lu"
#define SCNd64 "Ld"
#define SCNo64 "Lo"
#define SCNx64 "Lx"
#define SCNX64 "LX"
#endif

#endif				/*  __GFS_DOT_H__  */
