/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __LINUX_INCORE_DOT_H__
#define __LINUX_INCORE_DOT_H__

/*  Forward declarations for common stuff  */

struct gfs_bufdata_arch;
struct gfs_glock_arch;
struct gfs_inode_arch;
struct gfs_trans_arch;
struct gfs_sbd_arch;

/*  Definitions of common stuff  */

struct gfs_bufdata_arch {
	unsigned int bd_pinned;	/* Pin count */
	struct gfs_trans *bd_ail_tr;
	struct list_head bd_ail_tr_list;	/* List of buffers hanging off tr_ail_bufs */
	struct list_head bd_ail_gl_list;	/* List of buffers hanging off gl_ail_bufs */
};

struct gfs_glock_arch {
	struct inode *gl_aspace;
	struct list_head gl_dirty_buffers;
	struct list_head gl_ail_bufs;
};

struct gfs_inode_arch {
	struct task_struct *i_creat_task;
	long i_creat_pid;

	spinlock_t i_lock;
	struct buffer_head *i_cache[GFS_MAX_META_HEIGHT];

	struct list_head i_mmap_shared;
};

struct gfs_trans_arch {
	struct list_head tr_ail_bufs;
};

struct gfs_sbd_arch {
	spinlock_t sd_ail_lock;
	struct list_head sd_recovery_bufs;
};

#define gfs_get_transaction(sdp) ((struct gfs_trans *)current->journal_info)
#define gfs_set_transaction(sdp, tr) do { current->journal_info = (tr); } while (0)

#define gfs_get_file(vfile) ((struct gfs_file *)((vfile)->private_data))
#define gfs_set_file(vfile, fp) do { (vfile)->private_data = (fp); } while (0)

#if 1
#define gfs_dir_hash2offset(h) (((uint64_t)(h)) >> 1)
#define gfs_dir_offset2hash(p) ((uint32_t)(((uint64_t)(p)) << 1))
#else
#define gfs_dir_hash2offset(h) (((uint64_t)(h)))
#define gfs_dir_offset2hash(p) ((uint32_t)(((uint64_t)(p))))
#endif

#define GFS_MAY_READ           MAY_READ
#define GFS_MAY_WRITE          MAY_WRITE
#define GFS_MAY_EXEC           MAY_EXEC
#define GFS_MAY_EXCEED         (8)
#define GFS_MAY_ADMIN          (16)
#define GFS_IS_FOWNER          (32)
#define GFS_MAY_WRITE_USER_EA  (64)

/*  Definitions of arch-specific stuff  */

#define gfs_get_user_cred() ((int *)1)

extern kmem_cache_t *gfs_glock_cachep;
extern kmem_cache_t *gfs_inode_cachep;
extern kmem_cache_t *gfs_bufdata_cachep;
extern kmem_cache_t *gfs_mh_cache_cachep;

extern int gfs_glock_in_aspace;

struct gfs_vma_kludge
{
	struct list_head vk_list;
	struct vm_area_struct *vk_vma;
};

#endif				/*  __LINUX_INCORE_DOT_H__  */
