/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

/*
 * sislogger.h
 * Requires: stdarg.h, stdlib.h
 *
 * reference:
 *  "Header Files"  - http://www.cs.umd.edu/users/cml/cstyle/indhill-cstyle.htm
 *  "Include files" - http://www.cs.umd.edu/users/cml/cstyle/pikestyle.html
 * for details on why header files must be included by the user.
 */

#ifndef __SISLOGGER_H__
#define __SISLOGGER_H__

/* max length of the log msg prefix */
#define MAX_LOGPREFIX 31

/* log levels - correspond to syslog levels*/
#define LOG_DEBUG  7
#define LOG_INFO   6
#define LOG_NOTICE 5
#define LOG_WARN   4
#define LOG_ERR    3
#define LOG_CRIT   2
#define LOG_NOTHING 1

struct liblog_data;

/**
 * init_defaultlog - simpler initializer of the log - no syslog, no log file
 * @verbose:
 * @quiet:
 * @msgprefix:
 *
 * Description: don't log anything to syslog or a logfile, but get the
 * rest of the parameters from the caller
 *
 * Returns: as init_sislog
 */
#define init_defaultlog(verbose, quiet, msgprefix) \
        init_sislog(verbose, quiet, LOG_NOTHING, 0, msgprefix, NULL)

/**
 * init_minlog - Really simple initializer
 *
 * nothing to syslog, only warnings and errors printed
 *
 * Returns: as init_sislog
 */
#define init_minlog() init_sislog(0, 0, LOG_NOTHING, 0, "", NULL)

/*
 * plog - Printing helper macro
 */
#define plog(lvl, format...) \
do { \
        print_sislog(lvl, __FILE__, __LINE__,  ## format); \
} while(0)

/**
 * log_debug - debug level log macro. printf style syntax
 */
#define log_debug(format...) \
do { \
        plog(LOG_DEBUG, format); \
} while(0)

/**
 * log_info - information level log macro. printf style syntax
 */
#define log_info(format...) \
do { \
        plog(LOG_INFO, format); \
} while(0)

/**
 * log_notice - notice level log macro. printf style syntax
 */
#define log_notice(format...) \
do { \
        plog(LOG_NOTICE, format); \
} while(0)

/**
 * log_warn - warning level log macro. printf style syntax
 */
#define log_warn(format...) \
do { \
        plog(LOG_WARN, format); \
} while(0)

/**
 * log_err - error level log macro. printf style syntax
 */
#define log_err(format...) \
do { \
        plog(LOG_ERR, format); \
} while(0)

/**
 * log_crit - critical level log macro. printf style syntax
 */
#define log_crit(format...) \
do { \
        plog(LOG_CRIT, format); \
} while(0)

#define stack log_debug("<backtrace>")

int init_sislog(int verbose, int quiet, int debug, int syslog,
             const char *msg_prefix, const char *logfile);

void close_sislog(void);

void print_sislog(int level, char *file, int line, const char *format, ...);

#endif /* __SISLOGGER_H__ */


/*
 * Local variables:
 * c-file-style: "linux"
 * End:
 */
