/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_IOCTL_H__
#define __POOL_IOCTL_H__

#include "pool_std.h"		/* needed for constants (POOL_MAX_MULTIPATH, etc) */
/* IO Control Commands */

#define _POOLC_(x)      (('p' << 8) | x)

#define POOL_DUMP_TOTALSTATS    _POOLC_(0x00000031)
#define POOL_DUMP_DISKSTATS     _POOLC_(0x00000032)
#define POOL_CLEAR_STATS        _POOLC_(0x00000034)
#define POOL_READ_CAPACITY      _POOLC_(0x00000035)
#define POOL_DUMP_LOCKTAB       _POOLC_(0x00000036)
#define POOL_MEMEXP		_POOLC_(0x00000037)
#define POOL_DEBUG_ON		_POOLC_(0x00000038)
#define POOL_DEBUG_OFF		_POOLC_(0x00000039)

#define POOL_IND_CACHE          _POOLC_(0x00000050)
#define POOL_ASSEMBLE           _POOLC_(0x00000051)
#define POOL_REMOVE             _POOLC_(0x00000052)
#define POOL_GROW               _POOLC_(0x00000053)
#define POOL_COUNT              _POOLC_(0x00000054)
#define POOL_LIST               _POOLC_(0x00000055)
#define POOL_SPLIST             _POOLC_(0x00000056)
#define POOL_DEVLIST            _POOLC_(0x00000057)
#define POOL_GET_VERSION	_POOLC_(0x00000058)
#define POOL_MP_RESTORE		_POOLC_(0x00000059)
#define POOL_TUNE_MP		_POOLC_(0x00000060)
#define POOL_READCAPACITY       DIOCREADCAPACITY

typedef struct pool_devstatus {
	uint64_t pds_blocks;
	uint32_t pds_num_paths;
	uint32_t pds_num_good_paths;
	uint32_t pds_dev[POOL_MAX_MULTIPATH];
} pool_devstatus_t;

typedef struct pool_devlist {
	uint32_t pdl_count;
	uint32_t dummy;		/* alignment safety */
	pool_devstatus_t *pdl_list;
} pool_devlist_t;

typedef struct pool_spstatus {
	uint64_t sps_start;
	uint64_t sps_len;
	uint32_t sps_stripe;
	uint32_t sps_type;
	uint32_t sps_numdevs;
	uint32_t dummy;		/* alignment safety */
	pool_devstatus_t *sps_devlist;
} pool_spstatus_t;

typedef struct pool_splist {
	uint32_t spl_count;
	uint32_t dummy;		/* alignment safety */
	pool_spstatus_t *spl_list;
} pool_splist_t;

typedef struct pool_status {
	uint64_t ps_pool_id;
	uint64_t ps_capacity;
	uint32_t ps_pool_major;
	uint32_t ps_pool_minor;
	uint32_t ps_in_use;
	uint32_t ps_num_spools;	/* number of subpools in the pool */
	uint32_t ps_mptype;
	uint32_t ps_mpstripe;
	char ps_name[POOL_NAME_SIZE];
	pool_spstatus_t *ps_splist;
} pool_status_t;

typedef struct pool_list {
	uint32_t pl_count;
	uint32_t dummy;		/* alignment safety */
	pool_status_t *pl_list;
} pool_list_t;

typedef struct pool_stat {
	uint32_t s_reads;	/* # reads */
	uint32_t s_readblks;	/* # blocks reads */
	uint32_t s_writes;	/* # writes */
	uint32_t s_writeblks;	/* # blocks written */
} pool_stat_t;

#endif				/* __POOL_IOCTL_H__ */
