/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_ONDISK_H__
#define __POOL_ONDISK_H__
#include "pool_std.h"		/* needed for POOL_NAME_SIZE */

/* Pool label is at the head of every pool disk partition */

#define SIZEOF_POOL_LABEL       (8192)

typedef struct pool_label_s {
	uint64_t pl_magic;	/* Pool magic number */
	uint64_t pl_pool_id;	/* Unique pool identifier */
	char pl_pool_name[POOL_NAME_SIZE];	/* Name of pool */
	uint32_t pl_version;	/* Pool version */
	uint32_t pl_subpools;	/* Number of subpools in this pool */
	uint32_t pl_sp_id;	/* Subpool number within pool */
	uint32_t pl_sp_devs;	/* Number of data partitions in this subpool */
	uint32_t pl_sp_devid;	/* Partition number within subpool */
	uint32_t pl_sp_type;	/* Partition type */
	uint64_t pl_blocks;	/* Number of blocks in this partition */
	uint32_t pl_striping;	/* Striping size within subpool */
	/* If the number of DMEP devices is zero, then the next field **
	 ** (pl_sp_dmepid) becomes the subpool ID for redirection.  In **
	 ** other words, if this subpool does not have the capability  **
	 ** to do DMEP, then it must specify which subpool will do it  **
	 ** in it's place                                              */

	/* While the next 3 field are no longer used, they must stay to keep **
	 ** backward compatibility........................................... */
	uint32_t pl_sp_dmepdevs;	/* Number of dmep devices in this subpool */
	uint32_t pl_sp_dmepid;	/* Dmep device number within subpool */
	uint32_t pl_sp_weight;	/* if dmep dev, pref to using it */

	uint32_t pl_minor;	/* the pool minor number */
	uint32_t pl_padding;	/* reminder - think about alignment */

	/* Even though we're zeroing out 8k at the front of the disk before
	   writing the label, putting this in */
	char pl_reserve[184];	/* bump the structure size out to 512 bytes */
} pool_label_t;

void pool_label_out(pool_label_t * pl, char *buf);
void pool_label_in(pool_label_t * pl, char *buf);

#endif				/* __POOL_ONDISK_H__ */
