/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**  
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>

#include "ccs.h"

int using_ccs;
ccs_node_t *cluster_cn = NULL, *fence_cn = NULL;
ccs_node_t *nodes_cn = NULL;


static int open_files(char *dir_name, int verbose){
  if(!dir_name){
    int err=0;
    char *file_data=NULL;

    using_ccs=1;
    if(verbose) printf("Checking for appropriate files         :  ");
    if((err = get_ccs_file(&file_data, "list_files")) && (err < 0)){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr,
	      "Unable to get file list from CCS: %s\n", strerror(-err));
      return -1;
    }

    /* cluster.ccs */
    if(!strstr(file_data, "cluster.ccs\n")){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "cluster.ccs was not found.\n");
      free(file_data);
      return -1;
    }    
    /* nodes.ccs */
    if(!strstr(file_data, "nodes.ccs\n")){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "nodes.ccs was not found.\n");
      free(file_data);
      return -1;
    }    
    /* fence.ccs */
    if(!strstr(file_data, "fence.ccs\n")){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "fence.ccs was not found.\n");
      free(file_data);
      return -1;
    }    
    free(file_data);

    if((err = open_ccs_file(&cluster_cn, "cluster.ccs"))){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "Unable to open cluster.ccs: %s\n", strerror(-err));
      return -1;
    }

    if((err = open_ccs_file(&nodes_cn, "nodes.ccs"))){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "Unable to open nodes.ccs: %s\n", strerror(-err));
      close_ccs_file(cluster_cn);
      return -1;
    }

    if((err = open_ccs_file(&fence_cn, "fence.ccs"))){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "Unable to open fence.ccs: %s\n", strerror(-err));
      close_ccs_file(cluster_cn);
      close_ccs_file(nodes_cn);
      return -1;
    }

    if(verbose) printf("SUCCESS\n");
    return 0;
  } else {
    int fd = -1;
    int size=0;
    char filename[256];
    char *filedata = NULL;
    struct stat stat_buf;

    using_ccs=0;
    sprintf(filename, "%s/cluster.ccs", dir_name);
    if(stat(filename, &stat_buf)){
      fprintf(stderr, "Unable to stat cluster.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(!stat_buf.st_size){
      fprintf(stderr, "cluster.ccs contains no data.\n");
      goto fail;
    } else if(size < stat_buf.st_size){
      size = stat_buf.st_size;
    }

    sprintf(filename, "%s/nodes.ccs", dir_name);
    if(stat(filename, &stat_buf)){
      fprintf(stderr, "Unable to stat nodes.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(!stat_buf.st_size){
      fprintf(stderr, "nodes.ccs contains no data.\n");
      goto fail;
    } else if(size < stat_buf.st_size){
      size = stat_buf.st_size;
    }

    sprintf(filename, "%s/fence.ccs", dir_name);
    if(stat(filename, &stat_buf)){
      fprintf(stderr, "Unable to stat fence.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(!stat_buf.st_size){
      fprintf(stderr, "fence.ccs contains no data.\n");
      goto fail;
    } else if(size < stat_buf.st_size){
      size = stat_buf.st_size;
    }

    size++;
    if(!(filedata = malloc(size))){
      fprintf(stderr, "\nUnable to allocate enough memory.\n");
      return -1;
    }
    
    memset(filedata, 0, size);
    sprintf(filename, "%s/cluster.ccs", dir_name);
    if((fd = open(filename, O_RDONLY)) < 0){
      fprintf(stderr, "Unable to open cluster.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(read(fd, filedata, size) < 0){
      fprintf(stderr, "Unable to read cluster.ccs: %s\n", strerror(errno));
      goto fail;
    }
    close(fd); fd = -1;
    if(parse_filedata(&cluster_cn, filedata)){
      fprintf(stderr, "Error while parsing cluster.ccs.\n");
      goto fail;
    }

    memset(filedata, 0, size);
    sprintf(filename, "%s/nodes.ccs", dir_name);
    if((fd = open(filename, O_RDONLY)) < 0){
      fprintf(stderr, "Unable to open nodes.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(read(fd, filedata, size) < 0){
      fprintf(stderr, "Unable to read nodes.ccs: %s\n", strerror(errno));
      goto fail;
    }
    close(fd); fd = -1;
    if(parse_filedata(&nodes_cn, filedata)){
      fprintf(stderr, "Error while parsing nodes.ccs.\n");
      goto fail;
    }

    memset(filedata, 0, size);
    sprintf(filename, "%s/fence.ccs", dir_name);
    if((fd = open(filename, O_RDONLY)) < 0){
      fprintf(stderr, "Unable to open fence.ccs: %s\n", strerror(errno));
      goto fail;
    }
    if(read(fd, filedata, size) < 0){
      fprintf(stderr, "Unable to read fence.ccs: %s\n", strerror(errno));
      goto fail;
    }
    close(fd); fd = -1;
    if(parse_filedata(&fence_cn, filedata)){
      fprintf(stderr, "Error while parsing fence.ccs.\n");
      goto fail;
    }

    free(filedata);
    if(verbose) printf("Checking for appropriate files         :  SUCCESS\n");
    return 0;

  fail:
    if(verbose) printf("Checking for appropriate files         :  FAILURE\n");
    if(filedata) free(filedata);
    if(fd >= 0) close(fd);
    if(cluster_cn) free_ccs_node(cluster_cn);
    if(fence_cn) free_ccs_node(fence_cn);
    if(nodes_cn) free_ccs_node(nodes_cn);
    return -1;
  }
}

/* cluster_name_check
 *
 * This function check for a cluster name in cluster.ccs.
 *
 * Returns: 0 on success, -1 on failure
 */
static int cluster_name_check(int verbose){
  const char *cluster_name;
  ccs_node_t *name_cn;

  if(verbose) printf("Checking for cluster name              :  ");

  if(!(name_cn = find_ccs_node(cluster_cn, "cluster/name", '/')) || 
     !name_cn->v){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "The cluster name is not specified, or cluster.ccs is "
	    "not in the correct format.\n");
    return -1;
  }

  if(name_cn->v->type != CCS_STRING){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "The cluster name must be specified as a string.\n");
    return -1;
  }
  cluster_name = find_ccs_str(name_cn, "name", '/', "invalid_cluster_name");
  if(strlen(cluster_name) > 16 || strlen(cluster_name) < 1){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "The cluster name must be between 1 and 16 characters.\n");
    return -1;
  }

  if(verbose) printf("SUCCESS\n");
  return 0;
}


/* node_resolution_check
 *
 * This function ensures that the node names can be resolved.
 *
 * Returns: 0 on success, -1 on failure
 */
static int node_resolution_check(int verbose){
  int error = 0;
  ccs_node_t *node;
  struct hostent *he;

  for(node = nodes_cn->child; node; node = node->sib){
    he = gethostbyname(node->key);
    if(!he){
      error = -1;
      fprintf(stderr, "Unable to resolve node name \"%s\"\n", node->key);
    }
  }
  return error;
}


/* node_duplicate_check
 *
 * This function looks over the nodes.ccs file checking for duplicate
 * node names or duplicate IP addresses.
 *
 * Returns: 0 on success, -1 on failure
 */
static int node_duplicate_check(int verbose){
  ccs_node_t *node;

  if(verbose) printf("Checking for node name/IP duplicates   :  ");
  if(strcmp(nodes_cn->key, "nodes")){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "The main section keyword, \"nodes\", was not found in nodes.ccs.\n");
    goto fail;
  }

  node = nodes_cn->child;
  if(!node){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "Either there are no nodes specified in nodes.ccs,\n"
	    " or the file is not in the correct format.\n");
    goto fail;
  }
  for(; node; node = node->sib){
    ccs_node_t *tmp;
    for(tmp = node->sib; tmp; tmp = tmp->sib){
      if(!strcmp(node->key, tmp->key)){
	if(verbose) printf("FAILURE\n");
	fprintf(stderr, "Duplicate node names found for %s.\n", tmp->key);
	goto fail;
      }
    }
  }

  if(verbose) printf("SUCCESS\n");
  return 0;

 fail:
  return -1;
}


/* cluster_lock_check
 *
 * This function checks to see if GULM (RLM/SLM) has been specified.  If so, it
 * will also check to see that the servers are listed in the nodes.ccs file.
 *
 * In the future, this will also include sanity checks for the DLM.
 *
 * Returns: 0 on success, -1 on failure
 */
static int cluster_lock_check(int verbose){
  ccs_node_t *server_list;
  ccs_value_t *server;
  ccs_node_t *node;
  int found;

  if(find_ccs_node(cluster_cn, "cluster/lock_gulm", '/')){  /* GULM is being used */
    if(verbose) printf("Checking for GULM servers in nodes.ccs :  ");

    if((server_list = find_ccs_node(cluster_cn, "cluster/lock_gulm/servers", '/'))){
      /* cross-check node.ccs with server list */
      if(!server_list->v){
	if(verbose) printf("FAILURE\n");
	fprintf(stderr, "No lock_gulm servers found in cluster.ccs\n");
	return -1;
      }
      for(server = server_list->v; server; server = server->next){
	found = 0;
	if(server->type != CCS_STRING){
	  if(verbose) printf("FAILURE\n");
	  fprintf(stderr, "Server name not of valid string type.\n");
	  return -1;
	}
	for(node = nodes_cn->child; node; node = node->sib){
	  if(!strcmp(node->key, server->v.str)){ /* check name */
	    found = 1;
	  }
	  if(found)
	    break;
	}
	if(!found){
	  if(verbose) printf("FAILURE\n");
	  fprintf(stderr, "The lock_gulm server, \"%s\" was not found in nodes.ccs.\n",
		  server->v.str);
	  return -1;
	} else {
	  found = 0;
	}
      }
    } else {
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "The \"servers\" subsection for \"lock_gulm\" was not found in cluster.ccs\n");
      return -1;
    }
    if(verbose) printf("SUCCESS\n");
  }

  return 0;
}


/* fence_device_check
 *
 * This function check to make sure that there are fence devices
 * specified in fence.ccs, that they have an agent specified, and there
 * are no duplicate fence device names.
 *
 * Returns: 0 on success, -1 on failure
 */
static int fence_device_check(int verbose){
  int device_found=0;
  ccs_node_t *device_cn;

  if(verbose) printf("Checking for fence devices             :  ");

  if(!(device_cn = find_ccs_node(fence_cn, "fence_devices", '/'))){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "There is no 'fence_devices' section in fence.ccs.\n");
    goto fail;
  }

  for(device_cn = device_cn->child; device_cn; device_cn = device_cn->sib){
    ccs_node_t *tmp;
    for(tmp = device_cn->sib; tmp; tmp = tmp->sib){
      if(!strcmp(device_cn->key, tmp->key)){
	if(verbose) printf("FAILURE\n");
	fprintf(stderr, "There are duplicate entries for the fence device, "
		"\"%s\" in fence.ccs\n", device_cn->key);
	goto fail;
      }
    }
  }

  device_cn = find_ccs_node(fence_cn, "fence_devices", '/');

  for(device_cn = device_cn->child; device_cn; device_cn = device_cn->sib){
    int agent_found;
    ccs_node_t *tmp;

    for(agent_found=0, tmp=device_cn->child; tmp; tmp = tmp->sib){
      if(!strcmp(tmp->key, "agent")){
	if(agent_found){
	  if(verbose) printf("FAILURE\n");
	  fprintf(stderr, "Multiple agents specified for fence device, \"%s\" "
		  "in fence.ccs\n", device_cn->key);
	  goto fail;
	}else {
	  agent_found = 1;
	  device_found++;  /* count the number of devices w/ agents */
	}
      }
    }
    if(!agent_found){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "There is no agent specified for fence device, \"%s\" "
	      "in fence.ccs\n", device_cn->key);
      goto fail;
    }
  }

  if(!device_found){
    if(verbose) printf("FAILURE\n");
    fprintf(stderr, "No fencing devices were found in fence.ccs.\n");
    goto fail;
  }

  if(verbose) printf("SUCCESS\n");
  return 0;

 fail:
  return -1;
}


/* node_fence_cross_check
 *
 * This function makes sure that fence device referenced in nodes.ccs are
 * found in fence.ccs.
 *
 * Returns: 0 on success, -1 on failure
 */
static int node_fence_cross_check(int verbose){
  ccs_node_t *node;

  if(verbose) printf("Cross-checking nodes.ccs and fence.ccs :  ");

  for(node = nodes_cn->child; node; node = node->sib){
    ccs_node_t *fence_method;
    ccs_node_t *fence_dev;

    fence_method = node->child;
    fence_method = find_ccs_node(fence_method, "fence", '/');
    if(!fence_method || !fence_method->child){
      if(verbose) printf("FAILURE\n");
      fprintf(stderr, "There are no fencing methods specified for \"%s\" in "
	      "nodes.ccs.\n", node->key);
      goto fail;
    }

    for(fence_method = fence_method->child; fence_method;
	fence_method = fence_method->sib){
      char str[256];
      fence_dev = fence_method->child;
      if(!fence_dev){
	sprintf(str, "nodes/%s/fence/%s", node->key, fence_method->key);
	if(verbose) printf("FAILURE\n");
	fprintf(stderr, "There are no fence devices specified for the follow"
		"ing fence method in nodes.ccs:\n \"%s\".\n", str);
	goto fail;
      }
      sprintf(str, "fence_devices/%s", fence_dev->key);
      if(!find_ccs_node(fence_cn, str, '/')){
	if(verbose) printf("FAILURE\n");
	fprintf(stderr, "The fence device \"%s\" referenced in nodes.ccs "
		"was not found in fence.ccs.\n", fence_dev->key);
	goto fail;
      }
    }
  }
    

  if(verbose) printf("SUCCESS\n");
  return 0;

 fail:
  return -1;
}


/* test_ccs -
 * @dir_name:
 * @verbose:
 *
 * This function uses CCS to gain access to cluster information.
 * It then validates that all the necessary information is
 * available for a working GFS system.
 *
 * Returns: 0 on success, -1 on failure.
 */
int test_ccs(char *dir_name, int verbose){
  if(open_files(dir_name, verbose)){
    return -1;
  }

  if(cluster_name_check(verbose)){
    goto fail;
  }

  if(node_resolution_check(verbose)){
    goto fail;
  }

  if(node_duplicate_check(verbose)){
    goto fail;
  }

  /* check for gulm in cluster.ccs */
  if(cluster_lock_check(verbose)){
    goto fail;
  }

  if(fence_device_check(verbose)){
    goto fail;
  }

  if(node_fence_cross_check(verbose)){
    goto fail;
  }
  
  if(using_ccs){
    close_ccs_file(cluster_cn);
    close_ccs_file(nodes_cn);
    close_ccs_file(fence_cn);
  } else {
    free_ccs_node(cluster_cn);
    free_ccs_node(nodes_cn);
    free_ccs_node(fence_cn);
  }
  return 0;

 fail:
  if(using_ccs){
    close_ccs_file(cluster_cn);
    close_ccs_file(nodes_cn);
    close_ccs_file(fence_cn);
  } else {
    free_ccs_node(cluster_cn);
    free_ccs_node(nodes_cn);
    free_ccs_node(fence_cn);
  }
  return -1;
}
