#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "ccs_log.h"


int test_lockfile(char *lockfile){
  int fd;
  struct stat stat_buf;
  struct flock lock;
  
  if(!strncmp(lockfile, "/var/run/sistina/", 17)){
    if(stat("/var/run/sistina", &stat_buf)){
      if(mkdir("/var/run/sistina", S_IRWXU)){
	log_sys_err("Cannot create lockfile directory");
	return -1;
      }
    } else if(!S_ISDIR(stat_buf.st_mode)){
      log_err("/var/run/sistina is not a directory.\n"
	      "Cannot create lockfile.\n");
      return -1;
    }
  }

  if((fd = open(lockfile, O_CREAT | O_WRONLY,
		(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) < 0){
    log_sys_err("Cannot create lockfile");
    return -1;
  }

  lock.l_type = F_WRLCK;
  lock.l_start = 0;
  lock.l_whence = SEEK_SET;
  lock.l_len = 0;

  if (fcntl(fd, F_SETLK, &lock) < 0) {
    close(fd);
    log_err("The daemon is already running.\n");
    return -1;
  }

  unlink(lockfile);
  return 0;
}


int create_lockfile(char *lockfile){
  int fd;
  struct stat stat_buf;
  struct flock lock;
  char buffer[50];
  
  if(!strncmp(lockfile, "/var/run/sistina/", 17)){
    if(stat("/var/run/sistina", &stat_buf)){
      if(mkdir("/var/run/sistina", S_IRWXU)){
	log_sys_err("Cannot create lockfile directory");
	return -1;
      }
    } else if(!S_ISDIR(stat_buf.st_mode)){
      log_err("/var/run/sistina is not a directory.\n"
	      "Cannot create lockfile.\n");
      return -1;
    }
  }

  if((fd = open(lockfile, O_CREAT | O_WRONLY,
		(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) < 0){
    log_sys_err("Cannot create lockfile");
    return -1;
  }

  lock.l_type = F_WRLCK;
  lock.l_start = 0;
  lock.l_whence = SEEK_SET;
  lock.l_len = 0;

  if (fcntl(fd, F_SETLK, &lock) < 0) {
    close(fd);
    log_err("The daemon is already running.\n");
    return -1;
  }

  if (ftruncate(fd, 0) < 0) {
    close(fd);
    return -1;
  }

  sprintf(buffer, "%d\n", getpid());

  if(write(fd, buffer, strlen(buffer)) < strlen(buffer)){
    close(fd);
    unlink(lockfile);
    return -1;
  }

  /* leave fd open - rely on exit to close it */
  return 0;
}

void destroy_lockfile(char *lockfile){
  FILE *fp;
  int pid;

  fp = fopen(lockfile, "r");
  if(!fp) return;

  fscanf(fp, "%d", &pid);
  
  fclose(fp);

  if(pid == getpid())
    unlink(lockfile);
}
