/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**  
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "fence_log.h"

int test_lockfile(char *lockfile){
  int fd;
  struct stat stat_buf;
  struct flock lock;
  
  if(!strncmp(lockfile, "/var/run/sistina/", 17)){
    if(stat("/var/run/sistina", &stat_buf)){
      if(mkdir("/var/run/sistina", S_IRWXU)){
	log_err("Cannot create lockfile directory, /var/run/sistina\n");
	return -1;
      }
    } else if(!S_ISDIR(stat_buf.st_mode)){
      log_err("/var/run/sistina is not a directory.\n"
	      "Cannot create lockfile.\n");
      return -1;
    }
  }

  if((fd = open(lockfile, O_CREAT | O_WRONLY,
		(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) < 0){
    log_err("Cannot create lockfile, %s\n", lockfile);
    return -1;
  }

  lock.l_type = F_WRLCK;
  lock.l_start = 0;
  lock.l_whence = SEEK_SET;
  lock.l_len = 0;

  if (fcntl(fd, F_SETLK, &lock) < 0) {
    close(fd);
    log_err("Daemon is already running.\n");
    return -1;
  }

  unlink(lockfile);
  return 0;
}


int create_lockfile(char *lockfile){
  int fd;
  struct stat stat_buf;
  struct flock lock;
  char buffer[50];
  
  if(!strncmp(lockfile, "/var/run/sistina/", 17)){
    if(stat("/var/run/sistina", &stat_buf)){
      if(mkdir("/var/run/sistina", S_IRWXU)){
	log_err("\nCannot create lockfile directory, /var/run/sistina\n");
	return -1;
      }
    } else if(!S_ISDIR(stat_buf.st_mode)){
      log_err("\n/var/run/sistina is not a directory.\n"
	      "Cannot create lockfile.\n");
      return -1;
    }
  }

  if((fd = open(lockfile, O_CREAT | O_WRONLY,
		(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) < 0){
    log_err("\nCannot create lockfile, %s\n", lockfile);
    return -1;
  }

  lock.l_type = F_WRLCK;
  lock.l_start = 0;
  lock.l_whence = SEEK_SET;
  lock.l_len = 0;

  if (fcntl(fd, F_SETLK, &lock) < 0) {
    close(fd);
    log_err("\nDaemon is already running.\n");
    return -1;
  }

  if (ftruncate(fd, 0) < 0) {
    close(fd);
    return -1;
  }

  sprintf(buffer, "%d\n", getpid());

  if(write(fd, buffer, strlen(buffer)) < strlen(buffer)){
    close(fd);
    unlink(lockfile);
    return -1;
  }

  /* leave fd open - rely on exit to close it */
  return 0;
}

void destroy_lockfile(char *lockfile){
  FILE *fp;
  int pid;

  fp = fopen(lockfile, "r");
  if(!fp) return;

  fscanf(fp, "%d", &pid);
  
  fclose(fp);

  if(pid == getpid())
    unlink(lockfile);
}
