/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include "servd.h"
#include "libgulm.h"
#include "interfaced.h"

gulm_interface_p interface;
struct completion interface_wait;
int interface_status;

void
init_interface(void)
{
	interface = NULL;
	interface_status = INTERFACE_DEAD;
}

int
gnbd_core_login_reply(void *misc, u64 gen, u32 err, u32 rank, u8 state)
{

	if (err) {
		printe("Core login returned error (%d)\n", err);
		interface_status = INTERFACE_DYING;
	} else
		printd("gnbd_serv logged into local core\n");

	return err;
}

int
gnbd_core_logout_reply(void *misc)
{

	interface_status = INTERFACE_DYING;
	printd("gnbd_serv logged out of local core\n");
	return 0;
}

int
gnbd_core_error(void *misc, u32 err)
{
	printe("Got error code %d from gulm callback\n", err);
	return 0;
}

static lg_core_callbacks_t gnbd_serv_cb = {
	login_reply:gnbd_core_login_reply,
	logout_reply:gnbd_core_logout_reply,
	error:gnbd_core_error
};

int
gulm_msg_loop(void)
{
	int err = 0;

	while (interface_status == INTERFACE_ALIVE) {
		err = lg_core_handle_messages(interface, &gnbd_serv_cb, NULL);
		if (err) {
			printe("error handling gulm messages %d\n", err);
			break;
		}
	}

	return err;
}

int
gnbd_interfaced(void *unused)
{
	int err;

	gnbd_daemonize("gnbd_interfaced");
	complete(&interface_wait);

	err = gulm_msg_loop();

	lg_release(interface);
	interface = NULL;
	interface_status = INTERFACE_DEAD;
	complete(&interface_wait);
	return err;
}

int
start_interface(void)
{
	int err = 0;

	if (interface_status != INTERFACE_DEAD) {
		printd("gnbd_interfaced is already running\n");
		goto out;
	}
	init_completion(&interface_wait);
	err = lg_initialize(&interface, NULL, "GNBD Server Interface");
	if (err) {
		printe("couldn't initialize the interface (%d)\n", err);
		goto fail;
	}
	err = lg_core_login(interface, TRUE);
	if (err) {
		printe("couldn't login to lock_gulmd (%d). Is it running?\n",
		       err);
		goto interface_fail;
	}
	interface_status = INTERFACE_ALIVE;
	err = kernel_thread(gnbd_interfaced, NULL, 0);
	if (err < 0) {
		printe("couldn't start gnbd_interfaced process (%d)\n", err);
		goto logged_in_fail;
	}
	wait_for_completion(&interface_wait);

	return 0;

      logged_in_fail:
	lg_core_logout(interface);
	gulm_msg_loop();
      interface_fail:
	lg_release(interface);
      fail:
	interface = NULL;
      out:
	return err;
}

void
stop_interface(void)
{

	if (interface) {
		lg_core_logout(interface);
		wait_for_completion(&interface_wait);
	}

}

void
kill_interface(void)
{

	if (interface)
		lg_release(interface);

}
